
/*
 * $Header: /home/mb/cvs/rboot/rboot/src/meter/meter.c,v 1.2 1999/09/12 18:00:02 mb Exp $
 * Copyright (C) 1999 by Martin Butkus, Germany <m.butkus@tu-bs.de>
 *
 * Description: XXX
 */

/* Includes */

#include <stdio.h>
#include <sys/types.h>
#include <md5.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <limits.h>
#include <strings.h>

/* Typedef's */

/* Define's */

#define BLOCKSIZE (1*1024*1024)

/* Global variables */

extern char *optarg;
extern int optind;



/* Functions */

/* print usage info */

void usage(char *prgname) {
        fprintf(stderr,"Usage: %s [-m] bytes infile outfile outfile\n",prgname);
        exit(1);
}

/* print error msg and exit */

void fail(char *where, int what) {
        fprintf(stderr,"%s: %s\n",where, strerror(what));
        exit(1);
}


int main(int c, char **v) {
	int ch;

	char *prgname;
	int in, out1, out2;	/* file descriptors */

	int md5;
	off_t size;		/* how much to copy */
	
	MD5_CTX *context = NULL;

	/* inits */

	md5 = size = 0;
	prgname = *v;

        /* parse arguments */

        while ((ch = getopt(c, v, "m")) != -1) 
		switch(ch) {
		case 'm': md5 = 1; break;
		default: usage(prgname);
		}
	
        c -= optind; v += optind;

	if (c != 4) usage(prgname);

	size = strtouq(v[0], (char **)NULL, 10); 
	if (size == 0) usage(prgname);

	++v;	
	
	in=open(v[0],O_RDONLY); 
	if (in<0) fail(v[0], errno);

	out1=open(v[1],O_WRONLY|O_TRUNC|O_CREAT, 0666);
	if (out1<0) fail(v[1], errno);

	out2=open(v[2],O_WRONLY|O_TRUNC|O_CREAT, 0666);
	if (out2<0) fail(v[2], errno);

	if (md5) { 
                context = malloc(sizeof(MD5_CTX));
                if (context == NULL) fail(prgname,ENOMEM);      
                MD5Init(context);
        }       


	/* main loop */

	while (size > 0) {
		unsigned char buf[BLOCKSIZE];
		int sz;

		sz = (size < BLOCKSIZE) ? size : BLOCKSIZE;
		
		if (read(in,buf,sz) < sz) fail(v[0],errno);
		if (write(out1,buf,sz) < 0) fail(v[1],errno);
		if (write(out2,buf,sz) < 0) fail(v[2],errno);
		if (md5) MD5Update(context,buf,sz);

		putc('.',stderr); fflush(stderr);
		size -= sz;
	}
	
	close(in); close(out1); close(out2); 

        if (md5) {
                char md5_result[33];

                /* calculate and print the md5 digest */

                MD5End(context,md5_result);
                free(context);

                printf("%s\n",md5_result);
        }
	
	return 0;
}
